@@ -1,5 +1,5 @@
 /* EventManager.java -- event management and notification infrastructure
-   Copyright (C) 2005 Free Software Foundation
+   Copyright (C) 2005, 2006, 2007 Free Software Foundation
 
 This file is part of GNU Classpath.
 
@@ -39,6 +39,7 @@ exception statement from your version. */
 
 package gnu.classpath.jdwp.event;
 
+import gnu.classpath.jdwp.Jdwp;
 import gnu.classpath.jdwp.VMVirtualMachine;
 import gnu.classpath.jdwp.exception.InvalidEventTypeException;
 import gnu.classpath.jdwp.exception.JdwpException;
@@ -56,7 +57,7 @@ import java.util.Iterator;
  * 2) Filter event notifications from the VM
  * 
  * If an event request arrives from the debugger, the back-end will
- * call {@link #reqestEvent}, which will first check for a valid event.
+ * call {@link #requestEvent}, which will first check for a valid event.
  * If it is valid, <code>EventManager</code> will record the request
  * internally and register the event with the virtual machine, which may
  * choose to handle the request itself (as is likely the case with
@@ -69,7 +70,7 @@ import java.util.Iterator;
 public class EventManager
 {
   // Single instance
-  private static EventManager _instance = new EventManager ();
+  private static EventManager _instance = null;
 
   // maps event (EVENT_*) to lists of EventRequests
   private Hashtable _requests = null;
@@ -79,8 +80,11 @@ public class EventManager
    *
    * @return the event manager
    */
-  public static EventManager getDefault ()
+  public static EventManager getDefault()
   {
+    if (_instance == null)
+      _instance = new EventManager();
+
     return _instance;
   }
 
@@ -127,9 +131,10 @@ public class EventManager
     // only two: VM_INIT, VM_DEATH
     try
       {
+	byte sp = (Jdwp.suspendOnStartup()
+		   ? EventRequest.SUSPEND_THREAD : EventRequest.SUSPEND_NONE);
 	requestEvent (new EventRequest (0,
-					EventRequest.EVENT_VM_INIT,
-					EventRequest.SUSPEND_NONE));
+					EventRequest.EVENT_VM_INIT, sp));
 	requestEvent (new EventRequest (0,
 					EventRequest.EVENT_VM_DEATH,
 					EventRequest.SUSPEND_NONE));
