@@ -51,6 +51,7 @@ import java.util.Locale;
 
 import gnu.java.awt.font.FontDelegate;
 import gnu.java.awt.font.GNUGlyphVector;
+import gnu.java.awt.font.autofit.AutoHinter;
 import gnu.java.awt.font.opentype.truetype.TrueTypeScaler;
 import gnu.java.awt.font.opentype.truetype.Zone;
 
@@ -146,7 +147,8 @@ public final class OpenTypeFont
    */
   private GlyphNamer glyphNamer;
 
-  
+  private Hinter hinter;
+
   /**
    * Constructs an OpenType or TrueType font.
    *
@@ -579,6 +581,9 @@ public final class OpenTypeFont
                                                     FontRenderContext frc,
                                                     CharacterIterator ci)
   {
+    // Initialize hinter if necessary.
+    checkHinter(FontDelegate.FLAG_FITTED);
+
     CharGlyphMap cmap;    
     int numGlyphs;
     int[] glyphs;
@@ -689,13 +694,15 @@ public final class OpenTypeFont
                                                   float pointSize,
                                                   AffineTransform transform,
                                                   boolean antialias,
-                                                  boolean fractionalMetrics)
+                                                  boolean fractionalMetrics,
+                                                  int flags)
   {
     /* The synchronization is needed because the scaler is not
      * synchronized.
      */
+    checkHinter(flags);
     return scaler.getOutline(glyph, pointSize, transform,
-                             antialias, fractionalMetrics);
+                             antialias, fractionalMetrics, hinter, flags);
   }
 
   /**
@@ -837,4 +844,29 @@ public final class OpenTypeFont
     c[3] = (char) (tag & 0xff);
     return new String(c);
   }
+
+  /**
+   * Checks if a hinter is installed and installs one when not.
+   */
+  private void checkHinter(int flags)
+  {
+    // When another hinting impl gets added (maybe a true TrueType hinter)
+    // then add some options here. The Hinter interface might need to be
+    // tweaked.
+    if (hinter == null)
+      {
+        try
+          {
+            hinter = new AutoHinter();
+            hinter.init(this);
+          }
+        catch (Exception ex)
+          {
+            // Protect from problems inside hinter.
+            hinter = null;
+            ex.printStackTrace();
+          }
+      }
+    hinter.setFlags(flags);
+  }
 }
