@@ -1,5 +1,5 @@
 /* GtkFramePeer.java -- Implements FramePeer with GTK
-   Copyright (C) 1999, 2002, 2004 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2002, 2004, 2006, 2007 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -39,11 +39,9 @@ exception statement from your version. */
 package gnu.java.awt.peer.gtk;
 
 import java.awt.Frame;
-import java.awt.Graphics;
 import java.awt.Image;
 import java.awt.MenuBar;
 import java.awt.Rectangle;
-import java.awt.event.PaintEvent;
 import java.awt.peer.FramePeer;
 import java.awt.peer.MenuBarPeer;
 
@@ -59,6 +57,11 @@ public class GtkFramePeer extends GtkWindowPeer
   native void removeMenuBarPeer ();
   native void gtkFixedSetVisible (boolean visible);
 
+  private native void maximize();
+  private native void unmaximize();
+  private native void iconify();
+  private native void deiconify();
+
   int getMenuBarHeight ()
   {
     return menuBar == null ? 0 : getMenuBarHeight (menuBar);
@@ -122,25 +125,11 @@ public class GtkFramePeer extends GtkWindowPeer
 
   public void setBounds (int x, int y, int width, int height)
   {
-    // prevent window_configure_cb -> awtComponent.setSize ->
-    // peer.setBounds -> nativeSetBounds self-deadlock on GDK lock.
-    if (Thread.currentThread() == GtkToolkit.mainThread)
-      {
-        int menuBarWidth = width - insets.left - insets.right;
-        if (menuBar != null && menuBarWidth > 0)
-          setMenuBarWidthUnlocked (menuBar, menuBarWidth);
-
-        return;
-      }
-
     int menuBarWidth = width - insets.left - insets.right;
     if (menuBar != null && menuBarWidth > 0)
       setMenuBarWidth (menuBar, menuBarWidth);
 
-    nativeSetBounds (x, y,
-		     width - insets.left - insets.right,
-		     height - insets.top - insets.bottom
-		     + menuBarHeight);
+    super.setBounds(x, y, width, height + menuBarHeight);
   }  
   
   public void setResizable (boolean resizable)
@@ -187,75 +176,57 @@ public class GtkFramePeer extends GtkWindowPeer
 
   public void setIconImage (Image image) 
   {
-      if (image != null)
-	{
-	  if (image instanceof GtkImage)
-	    nativeSetIconImage((GtkImage) image);
-	  else
-	    nativeSetIconImage(new GtkImage(image.getSource()));
-	}
-  }
-
-  public Graphics getGraphics ()
-  {
-    Graphics g;
-    if (GtkToolkit.useGraphics2D ())
-      g = new GdkGraphics2D (this);
-    else
-      g = new GdkGraphics (this);
-    g.translate (-insets.left, -insets.top);
-    return g;
-  }
-  
-  protected void postConfigureEvent (int x, int y, int width, int height)
-  {
-    int frame_width = width + insets.left + insets.right;
-    // Since insets.top already includes the MenuBar's height, we need
-    // to subtract the MenuBar's height from the top inset.
-    int frame_height = height + insets.top + insets.bottom - menuBarHeight;
-
-    if (frame_width != awtComponent.getWidth()
-        || frame_height != awtComponent.getHeight())
-      awtComponent.setSize(frame_width, frame_height);
-
-    int frame_x = x - insets.left;
-    // Likewise, since insets.top includes the MenuBar height, we need
-    // to add back the MenuBar height to the frame's y position.  If
-    // no MenuBar exists in this frame, the MenuBar height will be 0.
-    int frame_y = y - insets.top + menuBarHeight;
-
-    if (frame_x != awtComponent.getX()
-        || frame_y != awtComponent.getY())
+    if (image != null)
       {
-        // awtComponent.setLocation(frame_x, frame_y);
+        GtkImage gtkImage;
+        if (image instanceof GtkImage)
+          gtkImage = (GtkImage) image;
+	else
+	  gtkImage = new GtkImage(image.getSource());
+
+        if (gtkImage.isLoaded && ! gtkImage.errorLoading)
+          nativeSetIconImage(gtkImage);
       }
   }
 
-  protected void postMouseEvent(int id, long when, int mods, int x, int y, 
-				int clickCount, boolean popupTrigger)
+  protected void postConfigureEvent (int x, int y, int width, int height)
   {
-    super.postMouseEvent (id, when, mods, 
-			  x + insets.left, y + insets.top, 
-			  clickCount, popupTrigger);
-  }
+    if (menuBar != null && width > 0)
+      setMenuBarWidthUnlocked (menuBar, width);
 
-  protected void postExposeEvent (int x, int y, int width, int height)
-  {
-    if (!isInRepaint)
-      q().postEvent (new PaintEvent (awtComponent, PaintEvent.PAINT,
-                                   new Rectangle (x + insets.left, 
-                                                  y + insets.top, 
-                                                  width, height)));
+    // Since insets.top already includes the MenuBar's height, we need
+    // to subtract the MenuBar's height from the top inset.
+    int frame_height = height - menuBarHeight;
+
+    // Likewise, since insets.top includes the MenuBar height, we need
+    // to add back the MenuBar height to the frame's y position.  If
+    // no MenuBar exists in this frame, the MenuBar height will be 0.
+    int frame_y = y + menuBarHeight;
+
+    super.postConfigureEvent(x, frame_y, width, frame_height);
   }
 
   public int getState ()
   {
-    return 0;
+    return windowState;
   }
 
   public void setState (int state)
   {
-
+    switch (state)
+      {
+        case Frame.NORMAL:
+          if ((windowState & Frame.ICONIFIED) != 0)
+            deiconify();
+          if ((windowState & Frame.MAXIMIZED_BOTH) != 0)
+            unmaximize();
+          break;
+        case Frame.ICONIFIED:
+          iconify();
+          break;
+        case Frame.MAXIMIZED_BOTH:
+          maximize();
+      }
   }
 
   public void setMaximizedBounds (Rectangle r)
@@ -267,11 +238,7 @@ public class GtkFramePeer extends GtkWindowPeer
     // TODO Auto-generated method stub
     
   }
-  public void updateAlwaysOnTop()
-  {
-    // TODO Auto-generated method stub
-    
-  }
+
   public boolean requestWindowFocus()
   {
     // TODO Auto-generated method stub
