@@ -1,5 +1,5 @@
 /* Toolkit.java -- AWT Toolkit superclass
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007
    Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
@@ -40,6 +40,7 @@ exception statement from your version. */
 package java.awt;
 
 import gnu.classpath.SystemProperties;
+import gnu.java.awt.AWTUtilities;
 import gnu.java.awt.peer.GLightweightPeer;
 import gnu.java.awt.peer.headless.HeadlessToolkit;
 
@@ -52,6 +53,7 @@ import java.awt.dnd.peer.DragSourceContextPeer;
 import java.awt.event.AWTEventListener;
 import java.awt.event.AWTEventListenerProxy;
 import java.awt.event.KeyEvent;
+import java.awt.font.TextAttribute;
 import java.awt.im.InputMethodHighlight;
 import java.awt.image.ColorModel;
 import java.awt.image.ImageObserver;
@@ -61,6 +63,7 @@ import java.awt.peer.CanvasPeer;
 import java.awt.peer.CheckboxMenuItemPeer;
 import java.awt.peer.CheckboxPeer;
 import java.awt.peer.ChoicePeer;
+import java.awt.peer.DesktopPeer;
 import java.awt.peer.DialogPeer;
 import java.awt.peer.FileDialogPeer;
 import java.awt.peer.FontPeer;
@@ -87,6 +90,7 @@ import java.net.URL;
 import java.security.AccessController;
 import java.security.PrivilegedAction;
 import java.util.ArrayList;
+import java.util.Hashtable;
 import java.util.Map;
 import java.util.Properties;
 import java.util.StringTokenizer;
@@ -120,7 +124,8 @@ public abstract class Toolkit
   /** The toolkit properties. */
   private static Properties props = new Properties();
 
-  protected final Map desktopProperties = new Properties();
+  protected final Map<String,Object> desktopProperties = 
+    new Hashtable<String,Object>();
 
   protected final PropertyChangeSupport desktopPropsSupport
     = new PropertyChangeSupport(this);
@@ -144,6 +149,15 @@ public abstract class Toolkit
     awtEventListeners = new AWTEventListenerProxy[0];
   }
 
+  /**
+   * 
+   * @param target
+   * @return
+   * @throws HeadlessException
+   */
+  protected abstract DesktopPeer createDesktopPeer(Desktop target)
+    throws HeadlessException;
+  
   /**
    * Creates a peer object for the specified <code>Button</code>.
    *
@@ -799,12 +813,11 @@ public abstract class Toolkit
    */
   public boolean getLockingKeyState(int keyCode)
   {
-    if (keyCode != KeyEvent.VK_CAPS_LOCK
-        && keyCode != KeyEvent.VK_NUM_LOCK
-        && keyCode != KeyEvent.VK_SCROLL_LOCK)
-      throw new IllegalArgumentException();
-    
-    throw new UnsupportedOperationException();
+    if (AWTUtilities.isValidKey(keyCode))
+      throw new UnsupportedOperationException
+	("cannot get locking state of key code " + keyCode);
+
+    throw new IllegalArgumentException("invalid key code " + keyCode);
   }
 
   /**
@@ -983,8 +996,8 @@ public abstract class Toolkit
   /**
    * @since 1.3
    */
-  public DragGestureRecognizer
-    createDragGestureRecognizer(Class recognizer, DragSource ds,
+  public <T extends DragGestureRecognizer> T
+    createDragGestureRecognizer(Class<T> recognizer, DragSource ds,
                                 Component comp, int actions,
                                 DragGestureListener l)
   {
@@ -1271,7 +1284,8 @@ public abstract class Toolkit
   /**
    * @since 1.3
    */
-  public abstract Map mapInputMethodHighlight(InputMethodHighlight highlight);
+  public abstract Map<TextAttribute,?>
+    mapInputMethodHighlight(InputMethodHighlight highlight);
 
   /**
    * Initializes the accessibility framework. In particular, this loads the
