@@ -1,5 +1,5 @@
 /* ByteOrder.java -- 
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2004  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -38,28 +38,39 @@ exception statement from your version. */
 
 package java.nio;
 
-import gnu.classpath.Configuration;
-
 /**
- * @author Michael Koch
+ * @author Michael Koch (konqueror@gmx.de)
  * @since 1.4
  */
 public final class ByteOrder
 {
+  /**
+   * Constant indicating big endian byte order.
+   */
   public static final ByteOrder BIG_ENDIAN = new ByteOrder();
-  public static final ByteOrder LITTLE_ENDIAN  = new ByteOrder();
+
+  /**
+   * Constant indicating little endian byte order.
+   */
+  public static final ByteOrder LITTLE_ENDIAN = new ByteOrder();
 
   /**
    * Returns the native byte order of the platform currently running.
+   *
+   * @return the native byte order
    */
   public static ByteOrder nativeOrder()
   {
-    return (System.getProperty ("gnu.cpu.endian").equals("big")
+    // Let this fail with an NPE when the property is not set correctly.
+    // Otherwise we risk that NIO is silently working wrong.
+    return (System.getProperty("gnu.cpu.endian").equals("big")
             ? BIG_ENDIAN : LITTLE_ENDIAN);
   }
 
   /**
    * Returns a string representation of the byte order.
+   *
+   * @return the string
    */
   public String toString()
   {
