@@ -204,6 +204,7 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetCursorUnlocked
 {
   void *ptr;
   GtkWidget *widget;
+  GdkWindow *win;
   GdkCursorType gdk_cursor_type;
   GdkCursor *gdk_cursor;
 
@@ -255,16 +256,20 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetCursorUnlocked
     }
       
   widget = get_widget(GTK_WIDGET(ptr));
-
+  
+  win = widget->window;
+  if ((widget->window) == NULL)
+    win = GTK_WIDGET(ptr)->window;
+    
   if (image == NULL)
     gdk_cursor = gdk_cursor_new (gdk_cursor_type);
   else
     gdk_cursor
-      = gdk_cursor_new_from_pixbuf (gdk_drawable_get_display (widget->window),
+      = gdk_cursor_new_from_pixbuf (gdk_drawable_get_display (win),
 				    cp_gtk_image_get_pixbuf (env, image),
 				    x, y);
 
-  gdk_window_set_cursor (widget->window, gdk_cursor);
+  gdk_window_set_cursor (win, gdk_cursor);
   gdk_cursor_unref (gdk_cursor);
 
   /* Make sure the cursor is replaced on screen. */
@@ -334,6 +339,42 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetSensitive
   gdk_threads_leave ();
 }
 
+JNIEXPORT jboolean JNICALL
+Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetHasFocus
+(JNIEnv *env, jobject obj)
+{
+  void *ptr;
+  jboolean retval;
+
+  gdk_threads_enter ();
+
+  ptr = NSA_GET_PTR (env, obj);
+  
+  retval = GTK_WIDGET_HAS_FOCUS((GTK_WIDGET (ptr)));
+
+  gdk_threads_leave ();
+
+  return retval;
+}
+
+JNIEXPORT jboolean JNICALL
+Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetCanFocus
+(JNIEnv *env, jobject obj)
+{
+  void *ptr;
+  jboolean retval;
+
+  gdk_threads_enter ();
+
+  ptr = NSA_GET_PTR (env, obj);
+  
+  retval = GTK_WIDGET_CAN_FOCUS((GTK_WIDGET (ptr)));
+
+  gdk_threads_leave ();
+
+  return retval;
+}
+
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetRequestFocus
   (JNIEnv *env, jobject obj)
@@ -618,16 +659,16 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_setNativeBounds
   width = width < 0 ? 0 : width;
   height = height < 0 ? 0 : height;
 
-  if (GTK_IS_VIEWPORT (widget->parent))
-    gtk_widget_set_size_request (widget, width, height);
-  else
+  if (!(width == 0 && height == 0))
     {
-      if (!(width == 0 && height == 0))
-        {
-          gtk_widget_set_size_request (widget, width, height);
-          if (widget->parent != NULL)
-            gtk_fixed_move (GTK_FIXED (widget->parent), widget, x, y);
-        }
+      gtk_widget_set_size_request (widget, width, height);
+      /* The GTK_IS_FIXED check here prevents gtk_fixed_move being
+         called when our parent is a GtkScrolledWindow.  In that
+         case though, moving the child widget is invalid since a
+         ScrollPane only has one child and that child is always
+         located at (0, 0) in viewport coordinates. */
+      if (widget->parent != NULL && GTK_IS_FIXED (widget->parent))
+        gtk_fixed_move (GTK_FIXED (widget->parent), widget, x, y);
     }
 
   gdk_threads_leave ();
@@ -748,6 +789,20 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_gtkWidgetSetForeground
   gdk_threads_leave ();
 }
 
+JNIEXPORT void JNICALL
+Java_gnu_java_awt_peer_gtk_GtkComponentPeer_realize (JNIEnv *env, jobject obj)
+{
+  void *ptr;
+
+  gdk_threads_enter ();
+
+  ptr = NSA_GET_PTR (env, obj);
+
+  gtk_widget_realize (GTK_WIDGET (ptr));
+
+  gdk_threads_leave ();
+}
+
 JNIEXPORT void JNICALL
 Java_gnu_java_awt_peer_gtk_GtkComponentPeer_setVisibleNative
   (JNIEnv *env, jobject obj, jboolean visible)
@@ -792,30 +847,6 @@ Java_gnu_java_awt_peer_gtk_GtkComponentPeer_isEnabled
   return ret_val;
 }
 
-JNIEXPORT jboolean JNICALL 
-Java_gnu_java_awt_peer_gtk_GtkComponentPeer_isRealized
-  (JNIEnv *env, jobject obj)
-{
-  void *ptr;
-  jboolean ret_val;
-
-  gdk_threads_enter ();
-
-  ptr = NSA_GET_PTR (env, obj);
-
-  if (ptr == NULL)
-    {
-      gdk_threads_leave ();
-      return FALSE;
-    }
-
-  ret_val = GTK_WIDGET_REALIZED (get_widget(GTK_WIDGET (ptr)));
-
-  gdk_threads_leave ();
-
-  return ret_val;
-}
-
 JNIEXPORT jboolean JNICALL 
 Java_gnu_java_awt_peer_gtk_GtkComponentPeer_modalHasGrab
   (JNIEnv *env __attribute__((unused)), jclass clazz __attribute__((unused)))
