@@ -1,5 +1,5 @@
 /* gnu_java_nio_VMChannel.c -
-   Copyright (C) 2003, 2004, 2005, 2006  Free Software Foundation, Inc.
+   Copyright (C) 2003, 2004, 2005, 2006, 2007  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -43,7 +43,6 @@ exception statement from your version. */
 #include <config-int.h>
 
 #include <sys/types.h>
-#include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -67,6 +66,14 @@ exception statement from your version. */
 #include <fcntl.h>
 #endif /* HAVE_FCNTL_H */
 
+#if defined(HAVE_SYS_IOCTL_H)
+#define BSD_COMP /* Get FIONREAD on Solaris2 */
+#include <sys/ioctl.h>
+#endif
+#if defined(HAVE_SYS_FILIO_H) /* Get FIONREAD on Solaris 2.5 */
+#include <sys/filio.h>
+#endif
+
 #define CONNECT_EXCEPTION "java/net/ConnectException"
 #define IO_EXCEPTION "java/io/IOException"
 #define SOCKET_EXCEPTION "java/net/SocketException"
@@ -372,6 +379,19 @@ Java_gnu_java_nio_VMChannel_setBlocking (JNIEnv *env,
     }  
 }
 
+/* Return true if fd is in non-blocking mode. */
+static jboolean
+is_non_blocking_fd(jint fd)
+{
+  int opts;
+  opts = fcntl(fd, F_GETFL);
+  if (opts == -1)
+    {
+      /* Assume blocking on error. */
+      return 0;
+    }
+  return (opts & O_NONBLOCK) != 0;
+}
 
 JNIEXPORT jint JNICALL 
 Java_gnu_java_nio_VMChannel_read__ILjava_nio_ByteBuffer_2 (JNIEnv *env,
@@ -418,8 +438,21 @@ Java_gnu_java_nio_VMChannel_read__ILjava_nio_ByteBuffer_2 (JNIEnv *env,
   else if (result == -1)
     {
       buf.count = 0;
-      if (errno == EAGAIN) /* Non-blocking */
-        result = 0;
+      if (errno == EAGAIN)
+        {
+          if (is_non_blocking_fd(fd))
+            {
+              /* Non-blocking */
+              result = 0;
+            }
+          else
+            {
+              /* Read timeout on a socket with SO_RCVTIMEO != 0. */
+              JCL_release_buffer(env, &buf, bbuf, JNI_ABORT);
+              JCL_ThrowException(env, SOCKET_TIMEOUT_EXCEPTION, "read timed out");
+              return -1;
+            }
+        }
       else if (errno == EBADF) /* Bad fd */
         {
           JCL_release_buffer(env, &buf, bbuf, JNI_ABORT);
@@ -580,8 +613,21 @@ Java_gnu_java_nio_VMChannel_readScattering (JNIEnv *env,
   /* Handle the response */
   if (result < 0)
     {
-      if (errno == EAGAIN) /* Non blocking */
-        result = 0;
+      if (errno == EAGAIN)
+        {
+          if (is_non_blocking_fd(fd))
+            {
+              /* Non-blocking */
+              result = 0;
+            }
+          else
+            {
+              /* Read timeout on a socket with SO_RCVTIMEO != 0. */
+              JCL_cleanup_buffers(env, bi_list, vec_len, bbufs, offset, bytes_read);
+              JCL_ThrowException(env, SOCKET_TIMEOUT_EXCEPTION, "read timed out");
+              return -1;
+            }
+        }
       else if (errno == EBADF) /* Bad fd */
         {
           JCL_cleanup_buffers(env, bi_list, vec_len, bbufs, offset, bytes_read);
@@ -943,7 +989,17 @@ Java_gnu_java_nio_VMChannel_read__I (JNIEnv *env,
   errno = tmp_errno;
 
   if (-1 == ret)
-    JCL_ThrowException (env, IO_EXCEPTION, strerror (errno));
+    {
+      if (errno == EAGAIN && !is_non_blocking_fd(fd))
+        {
+          /* Read timeout on a socket with SO_RCVTIMEO != 0. */
+          JCL_ThrowException(env, SOCKET_TIMEOUT_EXCEPTION, "read timed out");
+        }
+      else
+        JCL_ThrowException (env, IO_EXCEPTION, strerror (errno));
+      return -1;
+    }
+  
   if (0 == ret)
     return -1;
 
@@ -1103,6 +1159,7 @@ Java_gnu_java_nio_VMChannel_connect (JNIEnv *env, jclass clazz __attribute__((un
       if (EINPROGRESS == errno)
         {
           fd_set wrfds;
+          FD_ZERO(&wrfds);
           FD_SET(fd, &wrfds);
           ret = cpnio_select (fd + 1, NULL, &wrfds, NULL, &timeo);
           if (ret == -1)
@@ -1227,6 +1284,7 @@ Java_gnu_java_nio_VMChannel_connect6 (JNIEnv *env, jclass clazz __attribute__((u
       if (EINPROGRESS == errno)
         {
           fd_set wrfds;
+          FD_ZERO(&wrfds);
           FD_SET(fd, &wrfds);
           ret = cpnio_select (fd + 1, NULL, &wrfds, NULL, &timeo);
           if (ret == -1)
@@ -1450,6 +1508,10 @@ Java_gnu_java_nio_VMChannel_accept (JNIEnv *env,
           case EWOULDBLOCK:
 #endif
           case EAGAIN:
+            if (!is_non_blocking_fd(fd))
+              {
+                JCL_ThrowException(env, SOCKET_TIMEOUT_EXCEPTION, "Accept timed out");
+              }
             /* Socket in non-blocking mode and no pending connection. */
             return -1;
           default:
@@ -1554,8 +1616,6 @@ Java_gnu_java_nio_VMChannel_open (JNIEnv *env,
   int nmode = 0;
   int ret;
   const char *npath;
-  mode_t mask = umask (0);
-  umask (mask);
 
   if ((mode & CPNIO_READ) && (mode & CPNIO_WRITE))
     nmode = O_RDWR;
@@ -1575,7 +1635,7 @@ Java_gnu_java_nio_VMChannel_open (JNIEnv *env,
 
 /*   NIODBG("path: %s; mode: %x", npath, nmode); */
 
-  ret = open (npath, nmode, 0777 & ~mask);
+  ret = open (npath, nmode, 0666);
 
 /*   NIODBG("ret: %d\n", ret); */
 
